#!/usr/bin/perl
#  Copyright 2001-2010 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Error' => 'Error',
	   'Management' => 'Management',
	   'Replace Existing' => 'Replace Existing',

	   'Upload File' => 'Upload File',
	   'Error Reading Record' => 'Error Reading Record',
	   'The file must be a .csv file!' => 'The file must be a .csv file!',
	   'Maximum File Upload size exceeded!' => 'Maximum File Upload size exceeded!',
	   'Cannot open file' => 'Cannot open file',
	   'Add' => 'Add',
	   'Staff' => 'Staff',
	   'Student' => 'Student',
	   'Continue' => 'Continue',
	   'User' => 'User',

	   );

my $self = 'ldapmanage.pl';

my $defaultuidnumber = 10000;
my $ldap_staffgid = 500;
my $ldap_studentgid = 400;


my $maxbufcount = 500; # 500k max; change as required


use DBI;
use CGI;
use Text::CSV_XS;
use Crypt::GeneratePassword qw(word);


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "../../etc/ldap.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my $csv = Text::CSV_XS->new( {binary => 1} );

# Print Page Header
print "$doctype\n<html><head><title>". $lex{'Import Staff'};
print "</title><link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n";
#print "<a href=\"$exppage\">". $lex{Export}. "</a> ]\n";

print "<center><h1>LDAP ". $lex{Management}. "</h1></center>\n";

if ( not $arr{page} ) {
    showStartPage();
} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectFields();
} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    fillFields();
}



#-------------
sub fillFields {
#-------------

    #foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $replacemode = $arr{replace};
    delete $arr{replace};

    my $select;
    if ( not $replacemode ) { 
	$select = qq{where ldap_uidnumber is NULL or 
			 ldap_uidnumber = '' or 
			 ldap_gidnumber is NULL or
			 ldap_gidnumber = '' or 
			 ldap_uid is NULL or ldap_uid = ''};
    }


    # Find the maximum uid number (both staff and student)
    my $maxuidnumber = $defaultuidnumber;  # set above at 10k.
    foreach my $table ( 'studentall', 'staff' ) {

	my $sth = $dbh->prepare("select max( ldap_uidnumber ) from $table");
	$sth->execute;
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $maxuid = $sth->fetchrow;
	if ( $maxuid > $maxuidnumber ) { $maxuidnumber = $maxuid; }
    }
    $maxuidnumber++; # next available number.

    print "Max: $maxuidnumber<br>\n";


    # Loop through all students, first
    my $sth = $dbh->prepare("select lastname, firstname, studnum, 
     ldap_uid, ldap_uidnumber, ldap_gidnumber from studentall $select");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    # Query to check which table to update
    my $sth1 = $dbh->prepare("select count(*) from student where studnum = ?");

    while ( my ( $lastname, $firstname, $studnum, $uid, $uidnumber, $gidnumber ) = 
	    $sth->fetchrow ) {

	# Find the table to update
	my $studenttable;
	$sth1->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $scount = $sth1->fetchrow;
	if ( $scount ) { 
	    $studenttable = 'student'; 
	} else {
	    $studenttable = 'studentwd';
	}


	my $newuid = user2uid( $lastname, $firstname, $studnum, 'student' );

	my %values = ();
	$values{uid} = $newuid;
	$values{uidnumber} = $maxuidnumber;
	$values{gidnumber} = $ldap_studentgidnumber;
	print "GID:$ldap_studentgidnumber<br>\n";


	if ( $replacemode ) { # then do updates to selected fields w/o checking for values.

	    foreach my $key ( keys %arr ) { # $key is each field value (uid, uidnumber, gidnumber)
		print "$firstname $lastname ($studnum )<br>\n";		
		my $field = 'ldap_'. $key;
		print "Field: $field  Key:$key Num: $maxuidnumber<br>\n";
		my $sth = $dbh->prepare("update $studenttable set $field = ? where studnum = ?");
		$sth->execute( $values{$key}, $studnum );
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
		if ( $key eq 'uidnumber' ) { $maxuidnumber++; } # ready for next
	    }
	} else { # update only blank values

	    if ( not $uid ) {
		my $sth = $dbh->prepare("update $studenttable set ldap_uid = ? where studnum = ?");
		$sth->execute( $values{uid}, $studnum ); 
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    }
	    if ( not $uidnumber ) {
		my $sth = $dbh->prepare("update $studenttable set ldap_uidnumber = ? where studnum = ?");
		$sth->execute( $values{uidnumber}, $studnum );
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
		$maxuidnumber++; # ready for next
	    }
	    if ( not $gidnumber ) {
		my $sth = $dbh->prepare("update $studenttable set ldap_gidnumber = ? where studnum = ?");
		$sth->execute( $values{gidnumber}, $studnum );
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    }
	}

    }

    print "</body></html>\n";

    exit;

}



#------------
sub user2uid {
#------------

    # convert staff info into a uid for ldap usage (which means on remote file server and Google Apps/Gmail )

    my ( $lastname, $firstname, $userid, $type ) = @_;  
    # type is either 'staff' or 'student'; $userid is studnum or staff userid

    if ( $type eq 'student' ) {
	my $uid = 's'. $userid;
	return $uid;
    } else {
	my $uid;
	if ( $userid ) {
	    $uid = $userid;
	} else { # make one
	    $fname = substr($firstname, 0, 1 ); # first initial
	    $uid = $fname. '.'. $lastname;
	    return $uid;
	}
    }
}




#----------------
sub showStartPage {
#----------------

    print "<center><h1>". $lex{Add}. " LDAP &ndash; ". $lex{Staff}. " &amp; ". $lex{Student}. "</h1>\n";

    #print "<div style=\"padding:0.4em;border:1px solid gray;\">\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";

    print "<tr><td align=\"right\">". $lex{'Replace Existing'};
    print "</td><td><input type=\"checkbox\" name=\"replace\" value=\"1\">\n";;
    print "</td></tr>\n";

    print "<tr><td align=\"right\"> ". $lex{Add};
    print " UID </td><td><input type=\"checkbox\" name=\"uidnumber\" value=\"1\" checked=\"checked\">\n";;
    print "</td></tr>\n";

    print "<tr><td align=\"right\"> ". $lex{Add};
    print " GID</td><td><input type=\"checkbox\" name=\"gidnumber\" value=\"1\" checked=\"checked\">\n";;
    print "</td></tr>\n";

    print "<tr><td align=\"right\"> ". $lex{Add}. q{ }. $lex{User};
    print "</td><td><input type=\"checkbox\" name=\"uid\" value=\"1\" checked=\"checked\">\n";;
    print "</td></tr>\n";

    print "<tr><td colspan=\"2\" align=\"center\"><input type=\"submit\" value=\"". $lex{Continue};
    print "\"></td></tr>\n";
    print "</form></table>\n";

    print "<hr width=\"50%\">\n";


    # File Upload Section
    print "<h1>". $lex{'Upload File'}. " - Not Developed Yet... </h1>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";

    # File Upload Section
    print "<form action=\"$self\" method=\"post\"  enctype=\"multipart/form-data\">\n";
    print "<input type=\"file\" name=\"filename\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<tr><td>". $lex{'Replace Existing'};
    print " <input type=\"checkbox\" name=\"replace\" value=\"1\"></td></tr>\n";

    print "<tr><td align=\"center\">";
    print "<input type=\"submit\" value=\"". $lex{'Upload File'}. "\">\n";
    print "</td></tr></form>\n";

    print "</table></center></body></html>\n";

    exit;

}



#---------------
sub selectFields {
#---------------

    foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    exit;

    my $file = $q->param("filename");
    my $name; my $ext; 
    my $filename = $file;  # fileName is output filename, file is input.

    if ( $file ) {

	$filename =~ s!^.*(\\|\/)!!; 
	$filename = lc($filename);
	@name = split /\./, $filename; # split on dots.
	$ext = $name[$#name];  # last element is the extension.
	unless ( $ext eq 'csv' ){
	    print "<b>". $lex{'The file must be a .csv file!'}. "</b>";
	    print "</body></html>\n";
	    die;
	}

	pop(@name); # pull off extension.
	foreach $n (@name){ $name .= "$n.";} # assemble name 
	chop; # remove trailing dot

	open ( OUTFILE, ">$filename") || 
	    die $lex{'Cannot open file'}. " $filename"; 
	my $bufcount = 0;
	while ( my $bytesread = read( $file, my $buffer, 1024) ) { 
	    print OUTFILE $buffer;
	    $bufcount++;
	    if ( $bufcount > $maxbufcount ) {
		print "<h1>". $lex{'Maximum File Upload size exceeded!'};
		print " ($maxbufcount K)</h1>\n";
		print "</body></html>\n";
		die $lex{'Maximum File Upload size exceeded!'};
	    }
	}

	close OUTFILE;

    } else {
	print $lex{'Cannot open file'};
	print "</body></html>\n";
    }

    # We should now have the file in place.
    # Open csv file for reading
    unless ( open ( FH,"<$filename" ) ) {
	print $lex{'Cannot open file'}. ": $!\n";
	die $lex{'Cannot open file'}. ": $!\n";
    }

    my @records = ();
    for my $count ( 1 .. $maxrecorddisplay ) {
	my $line = <FH>;
	if ( not $line ) { next; } # skip blank lines / empty records up to limit
	#print "LINE: $line<br>\n";
	if ( $csv->parse($line) ) {
	    my @fields = $csv->fields;
	    push @records, [ @fields ];
	} else { # Failure to parse
	    print $lex{'Error Reading Record'}. ":<br>$line<br>\n";
	    die;
	}
    }
    close FH;
    my $maxrecfields; # now find largest number of fields in any record
    for my $idx (0..$#records) {
	if ( $#{ $records[$idx] } > $maxrecfields ) {
	    $maxrecfields = $#{ $records[$idx] }
	}
    }

    #print "Field Index: $maxrecfields<br>\n"; # note zero based


    # Load the fieldnames and fieldvalues from meta.
    my $sth = $dbh->prepare("select fieldid, fieldname from meta 
      where tableid = 'staff' order by arrayidx");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    my @fields = ();
    while ( my ( $fieldid, $fieldname ) = $sth->fetchrow ) {
	$fieldname =~ s/\(//g; # strip open parens
	push @fields, "$fieldname ($fieldid)";
    }

    # values stored in staff_multi.
    push @fields, $lex{Grade}. " (grade)";
    push @fields, $lex{Homeroom}. " (homeroom)";
    push @fields, $lex{Position}. " (position)";


    # Now create the form for field name selection.
    print "<center><form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"filename\" value=\"$filename\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";

    print "<input type=\"submit\" value=\"". $lex{Continue}. "\">\n";
    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";

    foreach my $fieldcount ( 0 .. $maxrecfields ) { # from import values.

	print "<tr><td align=\"right\"><select name=\"fld". $fieldcount. "\"><option></option>\n";
	foreach my $fld ( @fields ) { print "<option>$fld</option>"; }
	print "</select></td>\n";

	# Now display imported records.
	for my $rec (0 .. 3) {
	    if ( $records[$rec]->[$fieldcount] ) {
		print "<td>". $records[$rec]->[$fieldcount]. "</td>\n";
	    }
	}
	print "</tr>\n";

    }

    print "</table>\n";
    print "</center></body></html>\n";

    exit;
}


