#!/usr/bin/perl
#  Copyright 2001-2015 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Update LDAP Users - change passwords and (similarly) lock/unlock accounts
#  Copy a command file to another server.


my %lex = ('Main' => 'Main',
	   'Error' => 'Error',
	   'Reset Directory Users' => 'Reset Directory Users',
	   'User' => 'User',
	   'Reset Password' => 'Reset Password',
	   'Unlock User' => 'Unlock User',
	   'Lock User' => 'Lock User',
	   'Continue' => 'Continue',
	   'Student' => 'Student',
	   'Staff' => 'Staff',
	   'Not Found' => 'Not Found',
	   'Password Reset' => 'Password Reset',
	   'Account Locked' => 'Account Locked',
	   'Nothing to do' => 'Nothing to do',


	   );

# now stored in ldap.conf
# my $slappasswd = '/usr/local/sbin/slappasswd';
# my $basedn = 'dc=jp2,dc=loccsd,dc=ca';
# my $servername = 'myserver.ca'; # address of the LDAP server to manage.

# my $adminuser = 'cn=admin,dc=jp2,dc=loccsd,dc=ca';
# my $adminpassword = 'password';

# my $group_staff = 'staff';
# my $group_student = 'student';
# my $org = 'jp2.loccsd.ca';


use DBI;
use CGI;
use Net::LDAP;
use Net::LDAP::Entry;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "../../etc/ldap.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = CGI->new;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

# Setup LDAP access
my $ldap = Net::LDAP->new( $servername ) or die "$@\n";
# bind to a directory with dn and password
my $mesg = $ldap->bind( $adminuser , password => $adminpassword );

# Open Database
my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


# Write HTML Header
print "$doctype\n<html><head><title>". $lex{'Reset Directory Users'}. "</title>\n";
print "<link rel=\"stylesheet\" type=\"text/css\" href=\"$css\">\n";
print "<style type=\"text/css\"> .cn { text-align:center;} .la {text-align:left;}</style>\n";
print "$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n";
#print "<a href=\"$exppage\">". $lex{Export}. "</a> ]\n";

print "<center><h1>". $lex{'Reset Directory Users'}. "</h1>\n";

if ( not $arr{page} ) {
    showStartPage();
} else {
    delete $arr{page};
    processUsers();
}


#----------------
sub showStartPage {
#----------------

    # Search LDAP for staff;
    my $searchbase = "ou=$group_staff,$basedn";
    $mesg = $ldap->search( # perform a search
			   base => $searchbase,
			   attrs => ['uid','sn','givenName'],
			   filter => "(& (uid=*) )",
			   );

    # check for errors
    $mesg->code && die $mesg->error;

    # Get list of ldap staff.
    my %ldapstaff = ();
    foreach my $entry ($mesg->entries) { 
	my $uid = $entry->get_value( 'uid' );
	my $sn = $entry->get_value( 'sn');
	my $givenname = $entry->get_value('givenName');
	$ldapstaff{$uid} = "$sn, $givenname";
    }
    # end of LDAP staff read


    # Search LDAP for students;
    my $searchbase = "ou=$group_student,$basedn";
    $mesg = $ldap->search( # perform a search
			   base => $searchbase,
			   attrs => ['uid','sn','givenName'],
			   filter => "(& (uid=*) )",
			   );

    # check for errors
    $mesg->code && die $mesg->error;

    # Get list of ldap students.
    my %ldapstudent = ();
    foreach my $entry ($mesg->entries) { 
	my $uid = $entry->get_value( 'uid' );
	my $sn = $entry->get_value( 'sn');
	my $givenname = $entry->get_value('givenName');
	$ldapstudent{$uid} = "$sn, $givenname";
    }

    $mesg = $ldap->unbind;   # take down session
    # end of LDAP read


    # Start Form
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    # Print Table heading
    print "<div style=\"font-weight:bold;font-size:120%;\">\n";
    print $lex{Student}. "</div>\n";


    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" style=\"float:left;\">\n";
    print "<tr><td colspan=\"4\" style=\"text-align:center;\">\n";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";

    print "<tr><th>". $lex{User}. "</th><th>". $lex{'Reset Password'}. "</th>\n<th>";
    print $lex{'Lock User'}. "</th></tr>\n";

    my $count = 1;

    # print student rows.
    foreach my $uid ( sort { $ldapstudent{$a} cmp $ldapstudent{$b} } keys %ldapstudent ) {

	my $name = $ldapstudent{ $uid };

	print "<tr><td class=\"la\">$name</td>\n";
	print "<td class=\"cn\"><input type=\"checkbox\" name=\"$uid:reset\" value=\"1\"></td>\n";
	print "<td class=\"cn\"><input type=\"checkbox\" name=\"$uid:lock\" value=\"1\"></td></tr>\n";
	
	if ( $count % 20 == 0 ) {
	    print "</table>\n";

	    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" style=\"float:left;\">\n";
	    print "<tr><td colspan=\"4\" style=\"text-align:center;\">\n";
	    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";

	    print "<tr><th>". $lex{User}. "</th><th>". $lex{'Reset Password'}. "</th>\n<th>";
	    print $lex{'Lock User'}. "</th></tr>\n";
	}
	$count++;


    }

    print "<tr><td colspan=\"4\" style=\"text-align:center;\"><input type=\"submit\" value=\"";
    print $lex{Continue}. "\"></td></tr>\n";

    print "</table></form></body></html>\n";

    exit;

}



#---------------
sub processUsers {
#---------------

    #foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    # Loop through all updates
    my $first = 1;
    foreach my $key ( sort keys %arr ) {

	my ( $uid, $action ) = split /:/, $key;

	if ( $first ) {
	    print "<table cellspacing = \"0\" border=\"1\" cellpadding=\"3\">\n";
	    $first = 0;
	}

	if ( $action eq 'reset' ) {
	    resetPassword( $uid );
	} elsif ( $action eq 'lock' ) {
	    lockAccount( $uid );
	} else {
	    print $lex{Error}. ": No Action Found<br>\n";
	}

	print "</table>\n";

    } # End of action Loop

    if ( $first ) { # nothing done...
	print "<p>". $lex{'Nothing to do'}. "</p>\n";
    }

    print "</body></html>\n";

    exit;

}



#----------------
sub resetPassword {
#----------------

    my $uid = shift;

    # Get Password from DB;
    my ( $password, $lastname, $firstname, $group );

    # Figure out What table... student or staff?
    foreach my $table ( qw( staff student )) {

	$passwordfield = 'password';
	if ( $table eq 'staff' ) { $passwordfield = 'password'; }

	my $sth = $dbh->prepare("select $passwordfield, lastname, firstname from $table where ldap_uid = ?");
	$sth->execute( $uid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	( $password, $lastname, $firstname ) = $sth->fetchrow;
	if ( $password ) { 
	    $group = $table;
	    last; 
	}
    }

    if ( not $password ) {
	print "<tr><td>". $lex{Error}. ": $uid ". $lex{'Not Found'}. "</td></tr>\n";
	return;
    }

    # Now reset the password in LDAP.
    my $entry = Net::LDAP::Entry->new;

    # Encrypt Password
    my $newpassword = `$slappasswd -h {md5} -s $password`;
    chomp $newpassword;

    # Set DN
    if ( $group eq 'staff' ) {
	$group = $group_staff;
    } else {
	$group = $group_student;
    }

    my $dn = "uid=$uid,ou=$group,$basedn";
    $entry->dn($dn);

    $entry->changetype(modify);

    $entry->replace ( userPassword => $newpassword );
    $entry->update( $ldap ); # update this entry.

    print "<tr><td>". $lex{'Password Reset'}. " $firstname $lastname ($uid)</td></tr>\n";
    
    return;

}



#--------------
sub lockAccount {
#--------------

    my $uid = shift;

    # Get LDAP record info.
    $mesg = $ldap->search( # perform a search
			   base => $basedn,
			   attrs => ['sn','givenName','userPassword', 'ou'],
			   filter => "(& (uid=$uid) )",
			   );

    # check for errors
    $mesg->code && die $mesg->error;
    my @entries = $mesg->entries;
    my $entry = pop @entries; # there should only be one!
 
    my $sn = $entry->get_value('sn');
    my $givenname = $entry->get_value('givenName');
    my $password = $entry->get_value('userPassword');
    my $ou = $entry->get_value('ou');

    my ($pwd1, $pwd2 ) = split( /\}/, $password );
    $pwd1 .= '}';
    $pwd2 = '!'. $pwd2;
    my $newpassword = $pwd1 . $pwd2;

    # print "PWD:$password<br>\n";
    # print "New: $newpassword<br>\n";
    # return;

    # Now reset the password in LDAP with new leading value.
    #my $entry = Net::LDAP::Entry->new;

    # Set DN
    my $dn = "uid=$uid,ou=$ou,$basedn";
    $entry->dn($dn);

    $entry->changetype( modify );

    $entry->replace ( userPassword => $newpassword );
   
    #print "<pre>\n";
    #print $entry->dump;
    #print "</pre>\n";

    $entry->update( $ldap ); # update this entry.

    print "<tr><td>". $lex{'Password Reset'}. q{:}. $lex{'Account Locked'};
    print " $givenname $sn ($uid)</td></tr>\n";

    return;
}

