#!/usr/bin/perl
#  Copyright 2001-2010 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Notes: We are using ldap_uid rather than staff userid, since they
# may not be identical. LDAP uses the uid as part of the Distinguished
# Name (dn). Thus all UID's are ldap_uid values.

my %lex = ('Main' => 'Main',
	   'Eoy' => 'Eoy',
	   'Add' => 'Add',
	   'Delete' => 'Delete',
	   'Checked' => 'Checked',
	   'Error' => 'Error',
	   'Synchronize Staff' => 'Synchronize Staff',
	   'Name' => 'Name',
	   'Select' => 'Select',
	   'Continue' => 'Continue',
	   'Present' => 'Present',
	   'Delete Staff from Directory' => 'Delete Staff from Directory',
	   'Add Staff to Directory' => 'Add Staff to Directory',
	   'Added' => 'Added',
	   'Deleted' => 'Deleted',
	   'Staff' => 'Staff',
	   'Present' => 'Present',
	   'No Staff Found' => 'No Staff Found',

	   );

my $self = 'syncstaff.pl';

# Now stored externally
# my $slappasswd = '/usr/local/sbin/slappasswd';
# my $basedn = 'dc=jp2,dc=loccsd,dc=ca';
# my $servername = 'myserver.mydomain.ca'; # address of the LDAP server to manage.

# my $adminuser = 'cn=admin,dc=jp2,dc=loccsd,dc=ca';
# my $adminpassword = 'password';

# my $group_staff = 'staff';
# my $group_student = 'student';
# my $org = 'jp2.loccsd.ca';


use DBI;
use CGI;
use Net::LDAP;
use Net::LDAP::Entry;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval require "../../etc/ldap.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

# Setup LDAP access
my $ldap = Net::LDAP->new( $servername ) or die "$@\n";
# bind to a directory with dn and password
my $mesg = $ldap->bind( $adminuser , password => $adminpassword );

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


# Print Page Header
print "$doctype\n<html><head><title>\n";
print $lex{'Add Ldap User'};
print "</title><link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";

print "$chartype\n</head><body>\n";
print "[ <a href=\"$homepage\">". $lex{Main}. "</a> |\n";
print "<a href=\"$eoypage\">". $lex{Eoy}. "</a> ]\n";
# Update this depending on final script locations.

print '<center><h1>'. $lex{'Synchronize Staff'}. "</h1>\n";

if ( not $arr{page} ) {
    showStartPage();
} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectStaff();
} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    updateLdap();
}


#----------------
sub showStartPage {
#----------------

    # Start the form
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<table cellpadding=\"3\" border=\"0\" cellspacing=\"0\">\n";

    print "<tr><td><input type=\"radio\" name=\"mode\" value=\"add\" checked=\"checked\">\n";
    print $lex{Add}. "</td><td><input type=\"radio\" name=\"mode\" value=\"delete\">\n";
    print $lex{Delete}. "</td></tr>\n";

    print "<tr><td>". $lex{Checked}. " <input type=\"checkbox\" name=\"chk\" value=\"1\"></td>\n";
    print "<td></td></tr>\n";

    print "<tr><td><input type=\"submit\" value=\"". $lex{Continue}. "\"></td>\n";
    print "<td></td></tr>\n";    

    print "</table></form></center></body></html>\n";

    exit;

} # end of showStartPage



#----------------
sub selectStaff {
#----------------

    my $mode = $arr{mode};
    my $chk;
    if ( $arr{chk} ) { $chk = q{checked="checked"}; }

    print "<div style=\"font-weight:bold;font-size:130%;\">\n";
    if ( $mode eq 'add' ) {
	print $lex{'Add Staff to Directory'};
    } else {
	print $lex{'Delete Staff from Directory'};
    }
    print "</div>\n";


    # Start the form
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";
    print "<input type=\"hidden\" name=\"mode\" value=\"$mode\">\n";
    print "<table cellpadding=\"3\" border=\"0\" cellspacing=\"0\">\n";

    print "<tr><td class=\"ra\"><input type=\"submit\" value=\"". $lex{Continue}. "\">";
    print "</td><td></td></tr>\n";

    # Search LDAP for staff uid.
    my $searchbase = "ou=$group_staff,$basedn";
    $mesg = $ldap->search( # perform a search
			   base => $searchbase,
			   attrs => ['uid','sn','givenName'],
			   filter => "(& (uid=*) )",
			   );

    # check for errors
    $mesg->code && die $mesg->error;

    # Get list of ldap staff.
    my %ldapstaff = ();
    foreach my $entry ($mesg->entries) { 
	my $uid = $entry->get_value( 'uid' );
	my $sn = $entry->get_value( 'sn');
	my $givenname = $entry->get_value('givenName');
	$ldapstaff{$uid} = "$sn, $givenname";
    }

    $mesg = $ldap->unbind;   # take down session
    # end of LDAP read

    if ( $mode eq 'add' ) {

	# Compare with local staff and find the missing.
	my $first = 1;
	my $sth = $dbh->prepare("select lastname, firstname, userid, ldap_uid from staff
          order by lastname, firstname");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	while ( my ( $lastname, $firstname, $userid, $ldapuid ) = $sth->fetchrow ) {
	    print "<tr><td><b>$lastname</b>, $firstname ($ldapuid)</td>\n<td>";
	    if ( not $ldapstaff{ $ldapuid } ) { # missing
		print "<input type=\"checkbox\" name=\"$ldapuid\" value=\"1\" ";
		print "$chk ></td></tr>\n";
		$first = 0;
	    } else {
		print $lex{Present}. "</td></tr>\n";
	    }
	}



    } else { # mode is delete

	# Show Local Staff to Delete
	my $first = 1;
#	my $sth = $dbh->prepare("select lastname, firstname, userid, ldap_uid from staff
#          order by lastname, firstname");
#	$sth->execute;
#	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
#	while ( my ( $lastname, $firstname, $userid, $ldapuid ) = $sth->fetchrow ) {

	foreach my $key ( sort { $ldapstaff{$a} cmp $ldapstaff{$b} } keys %ldapstaff ) {

	    print "<tr><td class=\"ra\">$ldapstaff{$key} ($key)</td>\n<td>";
	    print "<input type=\"checkbox\" name=\"$key\" value=\"1\" ";
	    print "$chk ></td></tr>\n";
	    $first = 0;

	}

	if ( $first ) {
	    print "<tr><td colspan=\"2\">". $lex{'No Staff Found'}. "</td>/tr>\n";
	}


    } # end of delete Mode

    print "<tr><td colspan=\"2\"><input type=\"submit\" value=\"". $lex{Continue}. "\">";
    print "</td></tr></table></form></center></body></html>\n";
	
    exit;


} # end of selectStaff



#-------------
sub updateLdap {
#-------------

    #foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $mode = $arr{mode};
    delete $arr{mode};

    my $sth = $dbh->prepare("select lastname, firstname, userid, ldap_uid, 
       ldap_uidnumber, ldap_gidnumber, password from staff where ldap_uid = ?");

    foreach my $uid ( sort keys %arr ) {

	if ( $mode eq 'add' ) {

	    my $entry = Net::LDAP::Entry->new;

	    # Get User Information.
	    $sth->execute( $uid );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my ( $lastname, $firstname, $userid, $ldapuid, $uidnumber, $gidnumber, $password ) 
		= $sth->fetchrow;

	    # Encrypt Password
	    my $newpassword = `$slappasswd -h {md5} -s $password`;
	    chomp $newpassword;

	    my $dn = "uid=$ldapuid,ou=$group_staff,$basedn";
	    $entry->dn($dn);

	    # Add the Entry
	    $entry->add(
			ou => 'staff',
			o => $org,
			cn => "$firstname $lastname",
			objectClass => 'person',
			objectClass => 'organizationalPerson',
			objectClass => 'inetOrgPerson',
			objectClass => 'posixAccount',
			objectClass => 'shadowAccount',
			objectClass => 'top',
			sn => $lastname,
			givenName => $firstname,
			uid => $ldapuid,
			uidNumber => $uidnumber,
			gidNumber => $gidnumber,
			gecos => "$firstname $lastname",
			loginShell => '/bin/bash',
			homeDirectory => "/home/$ldapuid",
			userPassword => "$newpassword"
			);


	    $entry->update( $ldap ); # update this entry.

	    print "$firstname $lastname ($userid)\n";
	    print " <b>". $lex{Added}. "</b><br>\n";

	} elsif ( $mode eq 'delete' ) { # do the delete.

	    my $entry = Net::LDAP::Entry->new;

	    my $dn = "uid=$uid,ou=$group_staff,$basedn";

	    $entry->dn($dn);

	    $entry->delete();
	    $entry->update( $ldap ); # update this entry.

	    print "$uid ". $lex{Deleted}. "\n";


	} # end of delete section.
    } # end user add loop

    print "</center></body></html>\n";

    exit;

} # end of updateLdap
