#!/usr/bin/perl
#  Copyright 2001-2015 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# enrolview.pl = view student enrollments by class or student
my $self = 'enrolview.pl';

my %lex = ('View' => 'View',
	   'Subject Enrollments' => 'Subject Enrollments',
	   'Top' => 'Top',
	   'Name' => 'Name',
	   'Course' => 'Course',
	   'Teacher' => 'Teacher',
	   'Term' => 'Term',
	   'Grade' => 'Grade',
	   'Subject' => 'Subject',
	   'Report Card' => 'Report Card',
	   'Main' => 'Main',
	   'Student' => 'Student',
	   'Sort' => 'Sort',
	   'Students' => 'Student',
	   'Missing' => 'Missing',
	   'Error' => 'Error',
	   'Not Found' => 'Not Found',

	   );


use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

# Assume that $arr{term} and $arr{order} is passed.
#  order can be subjsec or student

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;



# Print Page Header
my $title = "$lex{View} $lex{'Subject Enrollments'}";
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body><a name=\"top\"></a>\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> |\n";
print "<a href=\"$reppage\">$lex{'Report Card'}</a> ] \n";


# Find all of the term patterns.
my $sth = $dbh->prepare("select distinct startrptperiod, endrptperiod from subject
 order by startrptperiod, endrptperiod");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }


# Select/Sort form
print "<form action=\"$self\"  method=\"post\" style=\"display:inline;\">\n";
print "<select name=\"order\">";
if ( $arr{order} ) { 
    print "<option>$arr{order}</option>\n"; 
}
print "<option>$lex{Subject}</option><option>$lex{Student}</option></select>\n";

print "$lex{Term} <select name=\"term\"><option>$arr{term}</option>\n";
while ( my ( $startterm, $endterm ) = $sth->fetchrow ) {
    print "<option>$startterm-$endterm</option>";
}
print "</select><input type=\"submit\" value=\"$lex{Sort}\"></form>\n";


# Page Head
print "<h1>$title</h1>\n";

if ( not $arr{order} ) { # just stop for input.
    print "<body></html>\n";
    exit;
}

my $order = $arr{order};
my $select;
if ( $arr{term} ){
    my ($start, $end ) = split('-', $arr{term});
    $select = "and eval.term = '$arr{term}'";
}


if ( $order eq $lex{Subject} ){
    # order by subjcode, studnum

    $sth = $dbh->prepare("select s.lastname, s.firstname, 
     e.id, e.studnum, e.subjcode, s.grade, e.term 
     from eval e, student s 
     where s.studnum = e.studnum $select 
     order by e.subjcode, s.homeroom, s.lastname, s.firstname, e.term");

} else {  # order by student

    $sth = $dbh->prepare("select student.lastname, student.firstname, 
     eval.id, eval.studnum, eval.subjcode, student.grade, eval.term from eval, student 
     where student.studnum = eval.studnum $select 
     order by student.grade, student.homeroom, student.lastname, 
     student.firstname, eval.subjcode, eval.term");
}

$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $rows = $sth->rows;



if ( $order eq $lex{Subject} ){
    bySubject(); # print by subject
} else {
    byStudent(); # print by student
}

findUnenrolled( $arr{term} );

print "[ <a href=\"#top\">$lex{Top}</a> ]</body></html>\n";


#-----------------
sub findUnenrolled {
#-----------------

    my $term = shift;

    my $sth = $dbh->prepare("select lastname, firstname, studnum from student
      order by lastname, firstname");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

    my $sth1 = $dbh->prepare("select count(*) from eval where studnum = ? and term = ?");


    my $first = 1;
    while ( my ($lastname, $firstname, $studnum) = $sth->fetchrow ) {
	    
	$sth1->execute($studnum, $term);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth1->fetchrow;

	if ($count < 1) {
	    if ($first) {
		print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";
		print "<tr><th>$lex{Students} $lex{Missing} $lex{'Subject Enrollments'}</th></tr>\n";
		$first = 0;
	    }

	    print "<tr><td>$firstname $lastname ($studnum)</td></tr>\n";
	}

    } # End of while loop
    if (not $first) { print "</table><p></p>\n"; }
    

} # End of findUnenrolled



#------------
sub bySubject { # print by subject
#------------

    my $currcourse = -1;

    for ($i=1; $i <= $rows; ++$i) {
	my ($lastname, $firstname, $id, $studnum, $subjsec, $grade, $trm) 
          = $sth->fetchrow;
  
	$sth2 = $dbh->prepare("select s.description,s.teacher,st.lastname, st.firstname 
          from subject s, staff st where s.teacher = st.userid and subjsec = ?");
	$sth2->execute( $subjsec );
	if ($DBI::errstr){ print $DBI::errstr; die; }
	my ($description,$teacher, $tlname, $tfname ) = $sth2->fetchrow;
  
	if ($subjsec != $currcourse) {
	    if ( not $first ){ print "</table>";} else { $first = 0;}
	    print "<p></p>";
	    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	    print "<tr><th>$lex{Name}</th><th>$lex{Course}</th><th>$lex{Teacher}</th>";
	    print "<th>$lex{Term}</th></tr>\n"; 
	    $currcourse = $subjsec;
	}
	print "<tr><td><b>$lastname</b>, $firstname</td>";
	print "<td>$description ($subjsec)</td><td>$tlname, $tfname</td>\n";
	print "<td align=\"center\">$trm</td></tr>\n";
    }
    print "</table>\n";

} # End of bySubject



#------------
sub byStudent { # print by student
#------------

    my $currstud = '-1';
    my $currgrade = '-1';
    my $first = 1; # true
    print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";
    print "<tr><th>". $lex{Name}. "</th><th>". $lex{Course}. "</th><th>". $lex{Term}. "</th></tr>\n"; 

    $sth2 = $dbh->prepare("select s.description,s.teacher,st.lastname, st.firstname 
     from subject s, staff st where s.teacher = st.userid and subjsec = ?");

    for (1..$rows) {

	my ($lastname, $firstname, $id, $studnum, $subjsec, $grade, $trm) 
          = $sth->fetchrow;

	# Get Subject Name and Teacher
	$sth2->execute($subjsec);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my ($description,$teacher, $tlname, $tfname ) = $sth2->fetchrow;

	if ($studnum != $currstud) {
	    if (not $first){ 
		print "<tr><th colspan=\"3\" style=\"height:6px;\">";
		print "</td></tr>\n";
	    } else { 
		$first = 0;
	    }
	    $currstud = $studnum;
	}

	if ($currgrade ne $grade){
	    print "<tr><td colspan=\"3\" style=\"font-size:150%;";
	    print "color:white; background-color:#444; text-align:center;\">";
	    print "$lex{Grade} $grade</td></tr>\n";
	    $currgrade = $grade;
	}

	print "<tr><td><b>$lastname</b>, $firstname</td>";
	print "<td>$description ($tlname, $tfname)</td>\n";
	print "<td align=\"center\">$trm</td></tr>\n";
    }

    print "</table>\n";

} # End of byStudent
