#!/usr/bin/perl
#  Copyright 2001-2016 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Reset' => 'Reset',
	   'Subjects' => 'Subjects',
	   'Main' => 'Main',
	   'Report Card' => 'Report Card',
	   'Subject' => 'Subject',
	   'Teacher' => 'Teacher',
	   'Term' => 'Term',
	   'Grade' => 'Grade',
	   'Sort' => 'Sort by',
	   'No Teacher' => 'No Teacher',
	   'Subj-Sec' => 'Subj-Sec',
	   'Terms' => 'Terms',
	   'Edit' => 'Edit',
	   'Delete' => 'Delete',
	   'Clone' => 'Clone',
	   'SmDesc' => 'SmDesc',
	   'Enrol' => 'Enrol',
	   'Error' => 'Error',
	   'Field' => 'Field',
	   'Credit' => 'Credit',
	   'Difficulty' => 'Diff',
	   'Start Term' => 'Start Term',
	   'End Term' => 'End Term',
	   'Record(s) Updated' => 'Record(s) Updated',
	   'Contact' => 'Contact',
	   'Location' => 'Location',
	   'Course' => 'Course',
	   'Section' => 'Section',
	   'Master' => 'Master',

	   );


use DBI;
use CGI;
use Number::Format qw(:all);

my $maxterms = 12;

my $self = 'subjreset.pl';

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = CGI->new;
print $q->header( -charset, $charset );
my %arr = $q->Vars;




my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my $title = "$lex{Reset} $lex{Course} $lex{Master}";

print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$css" type="text/css">\n};
print qq{$chartype\n</head><body>\n};

print qq{[ <a href="$homepage">$lex{Main}</a> |\n};
print qq{<a href="$reppage">$lex{'Report Card'}</a> ]\n};


print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    updateRecords();
}



#----------------
sub showStartPage {
#----------------

    my %valueMap = ('teacher' => $lex{Teacher}, 'grade' => $lex{Grade}, 'term' => $lex{Term} );

    my $sortorder;
    if ( not $arr{sortorder} ) {
	$sortorder = 'grade'; #default
    } else {
	$sortorder = $arr{sortorder};
    }

    
    # Sorting Form
    print qq{\n<form action="$self" method="post" style="display:inline;">\n};
    print qq{<input type="submit" value="$lex{Sort}">\n};
    print qq{<select name="sortorder">\n};

    # Approach: functional but ugly. Find more elegant solution.
    if ( $arr{sortorder} ) {
	print qq{<option value="$arr{sortorder}">$valueMap{$sortorder}</option>\n};
    }
    if ( $arr{sortorder} ne 'grade' ) {
	print qq{<option value="grade">$lex{Grade}</option>};
    }
    if ( $arr{sortorder} ne 'teacher' ) {
	print qq{<option value="teacher">$lex{Teacher}</option>};
    }
    if ( $arr{sortorder} ne 'term' ) {
	print qq{<option value="term">$lex{Term}</option>};
    }

    print qq{</select>\n</form><hr>\n\n};
    
    
    # Load staff into hash; not restricted to classroom teachers.
    my (%teachers, %teachername);
    my $sth = $dbh->prepare("select lastname, firstname, userid from staff 
     order by lastname, firstname"); 
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    while ( my ( $lastname, $firstname, $userid) = $sth->fetchrow ) {
	$teachers{"$lastname, $firstname ($userid)"} = $userid;
	$teachername{$userid} = "$lastname,$firstname";
    }

    
    # Load Courses
    my %subjects;
    $sth = $dbh->prepare("select id, subjsec, description, grade, teacher, 
      startrptperiod, endrptperiod from subject"); 
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    while ( my ( $id, $subjsec, $description, $grade, $userid, $startterm, $endterm ) = $sth->fetchrow ) {

	if ( length( $grade ) ==  1 ) { $grade = '0'. $grade; }
	my $name = $teachername{$userid}; # get name from userid;
	
	if ( $sortorder eq 'teacher') {
	    $subjects{"$name$description$subjsec"} = $id;
	    
	} elsif ( $sortorder eq 'grade' ) {
	    $subjects{"$grade$description$subjsec"} = $id;
	    
	} elsif ( $sortorder eq 'term' ) {
	    $subjects{"$startterm$endterm$name$description$subjsec"} = $id;
	}
    }

    
    # Load Locations
    my @locations;
    $sth = $dbh->prepare("select distinct location from subject where location is not NULL and location != ''");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    while ( my $loc = $sth->fetchrow ) {
	push @locations, $loc;
    }
    @locations = sort @locations;
    
    
    # Start Form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};

    # Submit Button
    print qq{<div style="text-align:left;padding:0.4em;"><input type="submit" value="$lex{Reset}"></div>\n};

    # Start Table
    print qq{<table cellpadding="3" border="1" cellspacing="0">};

    # Table Heading
    print qq{<tr><th>$lex{Course} ($lex{Section})</th><th>$lex{Grade}</th><th>$lex{Teacher}</th>};
    print qq{<th>$lex{'Start Term'}</th>};
    print qq{<th>$lex{'End Term'}</th><th>$lex{Location}</th><th>$lex{Enrol}</th></tr>\n};


    # Queries setup
    my $sth1 = $dbh->prepare("select count(*) from eval where subjcode = ?");
    my $sth2 = $dbh->prepare("select lastname, firstname from staff where userid = ?");

    foreach my $key ( sort keys %subjects ) {

	my $id = $subjects{$key};

	# Get Subject Info
	my $sth = $dbh->prepare("select subjsec, description, grade, 
          teacher, startrptperiod, endrptperiod, location from subject where id = ?");
	$sth->execute( $id );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	my ( $subjsec, $description, $grade,  $userid, $startterm, $endterm, $location ) = $sth->fetchrow;

	# Get count of students enrolled in this subjects.
	$sth1->execute( $subjsec );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth1->fetchrow;

	my $terms = ($endterm - $startterm) + 1;
	if ($terms < 1) { $terms = 1; }
	my $enrollments = round( $count / $terms, 2);
	

	print qq{<tr><td class="la"><b>$description</b> ($subjsec)</td><td class="cn">$grade</td>\n};

	# Teacher
	print qq{<td><select name="$id:teacher"><option value="$userid">$teachername{$userid}</option>\n};
	foreach my $key ( sort keys %teachers ) {
	    print qq{<option value="$teachers{$key}">$key</option>};
	}
	print qq{\n<option></option></select></td>\n};
	
	# Terms
	print qq{<td><input type="text" name="$id:sterm" size="3" value="$startterm"></td>\n};
	print qq{<td><input type="text" name="$id:eterm" size="3" value="$endterm"></td>\n};

	# Location
	if ( @locations ) {
	    print qq{<td><select name="$id:location"><option>$location</option>};
	    foreach my $loc ( @locations ) {
		print qq{<option>$loc</option>\n};
	    }
	    print qq{<option></option></select></td>\n};
	    
	} else {
	    print qq{<td><input type="text" name="$id:location" size="12" value="$location"></td>\n};
	}
	    
	# Subject Enrollments
	print qq{<td>$enrollments</td></tr>\n};
	
    }

    print qq{</table>\n};

    # Submit Button
    print qq{<div style="text-align:left;padding:0.4em;"><input type="submit" value="$lex{Reset}"></div>\n};

    print qq{</form>\n};
    print qq{</body></html>\n};

    exit;

}


#---------------
sub updateRecords {
#---------------

    # foreach $key ( sort keys(%arr)) { print "K:$key V:$arr{$key}<br>\n"}
    
    delete $arr{sortorder}; # passed from sortorder form, not used here.

    
    foreach my $key ( sort keys %arr ) {

	my ($id, $fld ) = split(/:/, $key);

	# Check for start/end term errors:
	if ( $fld eq 'sterm' ) { #
	    if (( $arr{$key} > $maxterms ) or ( $arr{$key} < 0 )) {
		print "<h3>Error: $fld - $arr{$key}</h3>\n";
		print "</body></html>\n";
		exit;
	    }

	} elsif ( $fld eq 'eterm' ) { #
	    my $stermkey = $id. ':sterm';
	    my $stermval = $arr{$stermkey};
	    if (( $arr{$key} > $maxterms ) or ( $arr{$key} < 0 ) or ( $arr{$key} < $stermval )) {
		print "<h3>Error: $fld - $arr{$key}</h3>\n";
		print "</body></html>\n";
		exit;
	    }
	}

	if ( $fld eq 'eterm' ) { $fld = 'endrptperiod'; }
	if ( $fld eq 'sterm' ) { $fld = 'startrptperiod'; }


	my $sth = $dbh->prepare("update subject set $fld = ? where id = ?");
	$sth->execute( $arr{$key}, $id );
	if ($DBI::errstr ) { print "$DBI::errstr;<br>\n"; }

    }

    if ( not $DBI::errstr ) {
	print qq{<h3>$lex{'Record(s) Updated'}</h3></p>\n};
    } else {
	print qq{<h3>$lex{Error}: $DBI::errstr<br>\n};
	print qq{$lex{Contact} $adminname (<a href="mailto:$adminemail">$adminemail</a> )</h3>\n};
    }

    print qq{<p>[ <a href="$reppage">$lex{'Report Card'}</a> |\n};
    print qq{<a href="$self">$title</a> ]</p>\n};
    print qq{</body></html>\n};

    exit;

}
