#!/usr/bin/perl
#  Copyright 2001-2017 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Error' => 'Error',
	   'Evaluation Master Entry' => 'Evaluation Master Entry',
	   'Category' => 'Category',
	   'School' => 'School',	   
	   'Main' => 'Main',
	   'Objective' => 'Objective',
	   'Objectives' => 'Objectives',
	   'Add' => 'Add',
	   'Please select the student and objectives' => 
	     'Please select the student and objectives',
	   'Select Student' => 'Select Student',
	   'Save Objectives' => 'Save Objectives',
	   'Strategies, Resources and Activities' => 'Strategies, Resources and Activities',
	   'Responsible Person(s)' => 'Responsible Person(s)',
	   'Long Term Objective' => 'Long Term Objective',
	   'Note' => 'Note',
	   'You are limited to 32 objectives per &quot;subject&quot;' =>
	     'You are limited to 32 objectives per &quot;subject&quot;',
	   'Mastery' => 'Mastery',
	   'Term' => 'Term',	   
	   'Select' => 'Select',
	   'Save Objectives' => 'Save Objectives',
	   'Entry Error: No student selected or &gt; 32 objectives selected.' =>
	     'Entry Error: No student selected or &gt; 32 objectives selected.',
	   'Add More Eval Master Objectives' => 'Add More Eval Master Objectives',
	   'Contact' => 'Contact',
	   'Please record the following error' => 'Please record the following error',
	   'You already have an evaluation master for this subject' =>
	     'You already have an evaluation master for this subject',
	   'Mastery' => 'Mastery',
	   'Impact Area' => 'Impact Area',
	   'Please Log In' => 'Please Log In',
	   'Strategy' => 'Strategy',
	   'Target Skill' => 'Target Skill',
	   'Current Performance Level' => 'Current Performance Level',
	   'Evaluation Method' => 'Evaluation Method',
	   'Add' => 'Add',
	   'Student' => 'Student',
	   'Saved' => 'Saved',

	   );

my $self = 'iepobjectiveadd.pl';

use DBI;
use CGI;
use CGI::Session;

my $q = new CGI;
my %arr = $q->Vars;
my $studnum = $arr{studnum};

my @tim = localtime(time);
my $year = $tim[5] + 1900;
my $month = $tim[4] + 1;
my $currdate = "$year-$month-$tim[3]";


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');

    print $q->header;
    print "$doctype\n<html><head>\n";
    print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\"></head><body>\n";
    print "[ <a href=\"$tchpage\">$lex{Main}</a> ]\n";
    print "<form action=\"../tlogin.pl\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"userid\" value=\"$userid\">\n";
    print "<input type=\"hidden\" name=\"script\" value=\"iep/$self\">\n";
    print "<input type=\"submit\" value=\"$lex{'Please Log In'}\">\n";
    print "</form></body></html>\n";
    exit;

}

my $userid = $session->param('userid');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;

# Get User Name
my $sth = $dbh->prepare("select lastname, firstname from staff where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;

# Homerooms and Access for this teacher.
my (%homerooms, %access );
if ( $arr{page} == 2 ) { # get access checking ready

    # Get homeroom and check access credentials.
    my $sth = $dbh->prepare("select field_value from staff_multi where field_name = 'homeroom' and userid = ?");
    $sth->execute( $userid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $homeroom = $sth->fetchrow ) {
	$homerooms{$homeroom} = 1;
    }

    # Get Access credentials
    my $sth = $dbh->prepare("select field_value from staff_multi where field_name = 'access' and userid = ?");
    $sth->execute( $userid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $ac = $sth->fetchrow ) {
	$access{$ac} = 1;
    }
}


# Print Page Heading
my $title = "$lex{Add} $lex{Student} $lex{Objectives}";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\"></head>\n";
print "<body>[ <a href=\"$tchpage\">$lex{Main}</a> ] $firstname $lastname\n";

print qq{<form action="localoutcomeadd.pl" method="post" style="display:inline;">\n};
print qq{<input type="submit" value="Add New Objectives">\n};
print qq{</form>\n};

print "<h1>$title</h1>\n";

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectObjectives();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    writeRecord();
}



#----------------
sub showStartPage {
#----------------

    # Get objectives from table. 
    my $sth = $dbh->prepare("select distinct catnum, category, count(idnum) 
     from iep_objectives group by category order by category");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
    print qq{<tr><th>$lex{Category}</th><th>Objectives</th><th>Category<br>Number</th></tr>\n};

    while ( my ($catnum,$category, $count) = $sth->fetchrow ) {
	print qq{<tr><td class""><form action="$self" method="post">\n};
	print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
	print "<input type=\"hidden\" name=\"catnum\" value=\"$catnum\">\n";
	print "<input type=\"hidden\" name=\"studnum\" value=\"$arr{studnum}\">\n";
	print "<input type=\"submit\" value=\"$category\"></form>\n";
	print "<td>$count $lex{Objectives}</td><td>$catnum</td></tr>\n";
    }

    print "</table></body></html>\n";

    exit;

} # End of showStartPage


#-------------------
sub selectObjectives {
#-------------------

    my $catnum = $arr{catnum};
    my $currentstudnum = $arr{studnum};

    # Read in all iep students
    my $sth = $dbh->prepare("select lastname, firstname, studnum from iep_student");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    my ( $currentstud, %students );
    while ( my ( $lastname, $firstname, $studnum ) = $sth->fetchrow ) {
	if ( $studnum == $currentstudnum ) { 
	    $currentstud = "<b>$lastname</b>, $firstname ($studnum)";
	}
	my $studrec = "<b>$lastname</b>, $firstname ($studnum)";
	$students{$studrec} =  $studnum;
    }
    # Done loading students.

    # Now find subject name for entered value
    $sth = $dbh->prepare("select category from iep_objectives where catnum = ?");
    $sth->execute( $catnum );
    if ($DBI::errstr) { print $DBI::errstr; die;}    
    my $category = $sth->fetchrow;

    # Form Start
    print "<p>". $lex{'Please select the student and objectives'}. "</p>\n";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"catnum\" value=\"$catnum\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";

    # Student Select
    print qq{<table cellpadding="8" cellspacing="0" border="1">\n};
    print qq{<tr><td class="bcn">\n};
    print qq{$lex{Select} $lex{Student} };
    print qq{<select name="student"><option value="$currentstudnum">$currentstud</option>\n};
    foreach my $key ( sort keys %students ){
	print qq{<option value="$students{$key}">$key</option>\n};
    }
    print qq{</select></td></tr>};
    print qq{</table>\n};


    # Section subject specific stuff.
    # Now print objectives
    print "<h2>$category $lex{Objectives}</h2>\n";
    print "<div><b>$lex{Note}:</b> \n";
    print $lex{'You are limited to 32 objectives per &quot;subject&quot;'}. "<br>\n";
    print "<input type=\"submit\" value=\"$lex{'Save Objectives'}\"></div>\n";

    print "<table cellpadding=\"4\" cellspacing=\"0\" border=\"1\">\n";

    print "<tr><td colspan=\"4\"><b>". $lex{'Target Skill'}. "</b><br>\n";
    print "<textarea name=\"targetskill\" rows=\"2\" cols=\"100\"></textarea></td></tr>\n";

    print "<tr><td colspan=\"4\"><b>". $lex{'Current Performance Level'}. "</b><br>\n";
    print "<textarea name=\"currperform\" rows=\"2\" cols=\"100\"></textarea></td></tr>\n";

    print "<tr><td colspan=\"4\"><b>". $lex{'Strategies, Resources and Activities'}. "</b><br>\n";
    print "<textarea name=\"strategy\" rows=\"2\" cols=\"100\"></textarea></td></tr>\n";

    print "<tr><td colspan=\"4\"><b>". $lex{'Responsible Person(s)'}. "</b><br>\n";
    print "<input name=\"responsible\" size=\"100\"></td></tr>\n";

    print "<tr><td colspan=\"4\"><b>". $lex{'Evaluation Method'}. "</b><br>\n";
    print "<textarea name=\"evalmethod\" rows=\"2\" cols=\"100\"></textarea></td></tr>\n";

    print "<tr><td colspan=\"4\"><b>". $lex{'Long Term Objective'}. "</b><br>\n";
    print "<input type=\"text\" name=\"longtermobjective\" size=\"100\" ";
    print "maxlength=\"255\"></td></tr>\n";

    # Impact Areas
#    print "<tr><td colspan=\"4\"><b>". $lex{'Impact Area'}. "</b>\n";
#    print "<select name=\"impactarea\"><option></option>\n";
#    foreach my $impact ( @impacts ){ # from config file
#	print "<option>$impact</option>\n";
#    }
    #    print "</select></td></tr>\n";
    
    my @terms; # used below as starting term
    $sth = $dbh->prepare("select distinct term from eval order by term where term is not null and term != ''");
    $sth->execute;
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
    while ( my $term = $sth->fetchrow ) {
	push @terms, $term;
    }

    
    $sth = $dbh->prepare("select idnum, description from iep_objectives
      where catnum = ? order by idnum");
    $sth->execute( $catnum );
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }    


    print qq{<tr><th>$lex{Objective}</th><th>$lex{Mastery}</th><th>Start<br>$lex{Term}</th>};
    print qq{<th>$lex{Select}</th></tr>\n};

    while ( my ( $idnum, $description ) = $sth->fetchrow ) {

	print qq{<tr><td class="la"><i>$idnum</i>. $description</td>\n};

	# Mastery Level
	print qq{<td><select name="M-$idnum"><option></option>\n};
	foreach my $mst ( @g_MasteryLevel ) {
	    print qq{<option>$mst</option>};
	}
	print qq{</select></td>\n};

	# Starting Term for implementation
	print qq{<td><select name="T-$idnum"><option></option>\n};
	foreach my $trm ( @terms ) {
	    print qq{<option>$trm</option>};
	}
	print qq{</select></td>\n};


	print qq{<td class="cn">};
	print qq{<input type="checkbox" name="$idnum" value="1"></td></tr>\n};

    }

    print qq{</table><input type="submit" value="$lex{'Save Objectives'}">\n};
    print qq{</form></body></html>\n};

    exit;

} # End of addObjectives



#---------------
sub writeRecord {
#---------------

    # foreach my $key ( sort keys %arr  ) { print qq{K:$key V:$arr{$key}<br>\n}; }
    my $studnum = $arr{student};
    delete $arr{student};

    # Get the student's homeroom
    my $sth = $dbh->prepare("select homeroom from student where studnum = ?");
    $sth->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }  
    my $hr = $sth->fetchrow;

    # Check for access.
    if ( not $access{IEP} and not $homerooms{$hr} ){

	# Check if any grades have subject attendance, 
	# and check those grades for course enrollments
	my $sth = $dbh->prepare("select count(e.id) from eval e, subject s 
          where e.studnum = ? and e.subjcode = s.subjsec and s.teacher = ?");
	$sth->execute( $studnum, $userid );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }  
	my $count = $sth->fetchrow;
	if ( $count < 1 ) {
	    print qq{<h3>No Access Allowed</h3>\n};
	    print qq{</body></html>\n};
	    exit;
	}
    }


    # subject number.
    my $catnum = $arr{catnum};
    delete $arr{catnum};


    my @objarray;
    foreach my $key ( keys %arr ) {
	if ( $arr{$key} == 1 ) { # We have a selection for subject master
	    if ( $key =~ m/^T/ ) { next; } # skip Term 1 values
	    push @objarray, $key; 
	}
    }
    @objarray = sort { $a <=> $b } @objarray; # sort numerically.


    if ( $#objarray > 31 or not $studnum ) { # We have entry errors
	print "<h3>";
	print $lex{'Entry Error: No student selected or &gt; 32 objectives selected.'};
	print "</h3>\n";
	exit;
    }


    my $sth = $dbh->prepare("select count(*) from iep_evalmst where subnum = ? and studnum = ?");
    $sth->execute( $catnum, $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $mstcount = $sth->fetchrow;

    if ( $mstcount > 0 ){ # We have existing records...
	print "<h1>$lex{'You already have an evaluation master for this subject'}</h1>\n";
	print qq{</body></html>\n};
	exit;
    }


    # pull in objective description from the database.
    my $tmpcat = $dbh->quote( $catnum );
    my %rec; # hash to hold values to insert into evalmst table.
    $sth = $dbh->prepare("select description from iep_objectives where catnum = $tmpcat and idnum = ?");
    my $objcount = 1; # used to create 'obj1', 'obj2' keys

    foreach my $objective ( @objarray ) {  # $objective is an id number in subject.
	$sth->execute( $objective );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	my $desc = $sth->fetchrow;

	my $mastery = $arr{'M-'.$objective};
	if ( $mastery ){ $mastery = $lex{Mastery}. q{:}. $mastery; }
	my $term = $lex{Term}. ":$arr{'T-'.$objective}";
	delete $arr{'T-'. $objective};
	delete $arr{'M-'. $objective};

	my $value = $dbh->quote("($catnum-$objective) $desc $mastery $term");
	my $key = 'obj'. $objcount;
	$rec{$key} = $value;

	$objcount++;

    }

    # put in rest of record values
    $rec{targetskill} = $dbh->quote( $arr{targetskill} );
    $rec{currperform} = $dbh->quote( $arr{currperform} );
    $rec{evalmethod} = $dbh->quote( $arr{evalmethod} );
    $rec{strategy} = $dbh->quote( $arr{strategy} );

    $rec{responsible} = $dbh->quote( $arr{responsible} );
    $rec{longtermobjective} = $dbh->quote( $arr{longtermobjective} );
    $rec{impactarea} = $dbh->quote( $arr{impactarea} );
    $rec{studnum} = $dbh->quote( $studnum );
    $rec{subnum} = $dbh->quote( $catnum );

    my @keys = keys %rec;
    my $keys = join(',', @keys);
    my $values = join(',', ( values %rec ));

    $sth = $dbh->prepare("insert into iep_evalmst ( $keys ) values ( $values )");
    $sth->execute;

    if (not $DBI::errstr ) {
	print "<h3>$lex{Objectives} $lex{Saved}</h3>\n";

    } else {
	print "<h3>$lex{Error}: $DBI::errstr</h3>\n";
	print $lex{Contact}. " $adminname <a href=\"mailto:$adminemail\">$adminemail</a>\n";
    }

    print " <p>[ <a href=\"$iephome\">$lex{Main}</a> |\n";
    print "<a href=\"$self\">$lex{'Add More Eval Master Objectives'}</a> ]</p>\n";
    print "</body></html>\n";

    exit;


} # end of writeRecords
